#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import sys
import os
from commons.core.utils.RepetOptionParser import RepetOptionParser
from commons.core.checker.RepetException import RepetException
from commons.tools.CleanClusterNodesAfterRepet import CleanClusterNodesAfterRepet

REPET_PATH = os.environ.get("REPET_PATH")

def main():

    description = "Check MySQL connection on each node"
    usage = "CheckMysqlConnect.py [options]"
    examples = "\nExample 1: check MySQL connection from every node of a SGE cluster using environment variables:\n"
    examples += "\t$ python CheckMysqlConnect.py"
    examples += "\n\t"
    examples += "\nExample 2: check MySQL connection from every node of a SGE cluster using configuration file:\n"
    examples += "\t$ python CheckMysqlConnect.py -C configFileName"
    examples += "\n\t"
    examples += "\nExample 3: check MySQL connection from every node of a SGE cluster using environment variables and using a different python executable than the one in the PATH:\n"
    examples += "\t$ python CheckMysqlConnect.py -p /path/to/python"
    examples += "\n\n"
           
    parser = RepetOptionParser(description = description, usage = usage, version = "v1.0", epilog = examples)
    parser.add_option( "-p", "--path", dest = "pythonPath", type = "string", help = "path to python executable ('python' included, without finishing '/')", default = "python")
    parser.add_option( "-C", "--config", dest = "config", type = "string", help = "path to config file", default = "")

    options, args = parser.parse_args()
    pythonPath = options.pythonPath
    configFile =  options.config
    
    iCleanClusterNodesAfterRepet = CleanClusterNodesAfterRepet()
    nodesList = iCleanClusterNodesAfterRepet.getAllNodesList()
    
    nbNodes = len(nodesList)
    nodeCount = 0
    configOption = ""
    for node in nodesList:
        try:
            nodeCount += 1
            print "Connect to node '%s' (%i/%i)..." % (node, nodeCount, nbNodes)
            sys.stdout.flush()
            cmd = "ssh"
            cmd += " -q %s " % ( node )
            if configFile != "":
                configOption = " -C %s" % configFile
            cmd += "'%s %s/bin/MysqlConnect.py -n %s %s'" % (pythonPath, REPET_PATH, node, configOption)
            os.system(cmd)
        except RepetException, e:
            print e.getMessage()

    iCleanClusterNodesAfterRepet.clean()

if __name__ == '__main__':
    main()